/*------------------------------------------------------------------------------
ENDIAN.C

Zdrojovy kod bol upraveny pre potreby cviceni z predmetu APS
na demonstraciu testovania "endianity".


2018-11-6, M.D.
------------------------------------------------------------------------------*/

#include <REG52.H>                /* deklaracia specialnych funkcnych registrov */
                                  /* pre pouzity procesor na baze jadra 8051    */

#include <stdio.h>                /* deklaracia prototypov pre I/O funkcie      */


#define LITTLE_ENDIAN   0
#define BIG_ENDIAN      1

int machineEndianness( void )
{
   long int i = 1;
   const char *p = (const char *) &i;
   if (p[0] == 1)                  // nizsia adresa obsahuje LSB bajt
      return LITTLE_ENDIAN;
   else
      return BIG_ENDIAN;
}



/*------------------------------------------------
Funkcia main C. Vykon C programu zacina v tomto bode
po inicializacii zasobnika a dalsich registrov resp.
pamati.
Inicializacia je definova tzv. startovacim suborom,
ktory realizuje nevyhnutne inicializacie. V pripade
potreby je mozne tento subor modifikovat a doplnit
uzivatelom definovane inicializacie.
------------------------------------------------*/
void main (void) {

/*------------------------------------------------
Konfiguracia serioveho portu pre 1200 baudov a 16MHz krystale.
------------------------------------------------*/

/* ----------------------------------------------
Nasledujucich 5 riadkov zabezpeci vhodnu konfiguraciu perifernych
obvodov procesora Intl 8051, ktore su nevyhnutne pre seriovu
komunikaciu. Presny vyznam jednotlivych registrov bude analyzovany
pocas nasledujucich prednasok a cviceni.
*/

    SCON  = 0x50;               /* SCON: mod 1, 8-bitovy UART, povolenie prijmu             */
    TMOD |= 0x20;               /* TMOD: casovac (timer) 1, mod 2, 8-bitovy prepis (reload) */
    TH1   = 221;                /* TH1:  prepis hodnoty pre 1200 baudov @ 16MHz             */
    TR1   = 1;                  /* TR1:  spusti timer 1                                     */
    TI    = 1;                  /* TI:   nastavi TI s cielom vyslat z UARTu prvy znak       */



  if(machineEndianness() == LITTLE_ENDIAN) {
    P1 = 0xF0;                                /* Vynuluje bity P1.3-P1.0  */
    printf ("CPU je typu LITTLE_ENDIAN\n");   /* Vypise info na UART  */
  }
  else {
    P1 = 0x0F;                                /* Vynuluje bity P1.7-P1.4  */
    printf ("CPU je typu LITTLE_ENDIAN\n");   /* Vypise info na UART  */
  }
	

/*------------------------------------------------
Vsimnite si, ze program nikdy nekonci (pretoze nie je vyuzity ziadny
operacny system, ktoremu by bolo mozne odovzdat riadenie).
Nekonecna slucka je jeden zo sposobov, kotre su pri embedded aplikaciach
typicky vyuzivane.
------------------------------------------------*/

  while (1)              /* ukonci cinnost v nekonencnej slucke */
     ;
}


